Package: MOGAMUN
Type: Package
Title: MOGAMUN: A Multi-Objective Genetic Algorithm to Find Active
        Modules in Multiplex Biological Networks
Version: 1.23.0
Authors@R: 
    person("Elva-María", "Novoa-del-Toro", role=c("aut", "cre"),
    email="elvanov@hotmail.com", comment=c(ORCID = "0000-0002-6135-5839"))
Description: MOGAMUN is a multi-objective genetic algorithm that identifies
    active modules in a multiplex biological network. This allows analyzing
    different biological networks at the same time. MOGAMUN is based on
    NSGA-II (Non-Dominated Sorting Genetic Algorithm, version II), which we
    adapted to work on networks.
License: GPL-3 + file LICENSE
Encoding: UTF-8
Imports: stats, utils, RCy3, stringr, graphics, grDevices, RUnit,
        BiocParallel, igraph
Suggests: knitr, markdown
biocViews: SystemsBiology, GraphAndNetwork, DifferentialExpression,
        BiomedicalInformatics, Transcriptomics, Clustering, Network
URL: https://github.com/elvanov/MOGAMUN
RoxygenNote: 7.1.1
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/MOGAMUN
git_branch: devel
git_last_commit: 4e05af7
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:51:40 UTC; biocbuild
Author: Elva-María Novoa-del-Toro [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6135-5839>)
Maintainer: Elva-María Novoa-del-Toro <elvanov@hotmail.com>
