# Generated by roxygen2: do not edit by hand

export(mogamun_init)
export(mogamun_load_data)
export(mogamun_postprocess)
export(mogamun_run)
import(igraph)
import(stringr)
import(RUnit)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,MulticoreParam)
importFrom(RCy3,closeSession)
importFrom(RCy3,createColumnFilter)
importFrom(RCy3,createNetworkFromDataFrames)
importFrom(RCy3,createSubnetwork)
importFrom(RCy3,cytoscapePing)
importFrom(RCy3,layoutNetwork)
importFrom(RCy3,loadTableData)
importFrom(RCy3,saveSession)
importFrom(RCy3,setEdgeColorMapping)
importFrom(RCy3,setNodeBorderColorMapping)
importFrom(RCy3,setNodeBorderWidthMapping)
importFrom(RCy3,setNodeColorMapping)
importFrom(grDevices,dev.off)
importFrom(grDevices,rainbow)
importFrom(grDevices,svg)
importFrom(graphics,boxplot)
importFrom(graphics,legend)
importFrom(graphics,plot)
importFrom(stats,qnorm)
importFrom(stats,runif)
importFrom(utils,combn)
importFrom(utils,read.csv)
importFrom(utils,read.table)
importFrom(utils,write.csv)
importFrom(utils,write.table)
