\name{3_Coexpression_Sample.gr}
\docType{data}
\alias{3_Coexpression_Sample}
\title{Sample of interactions from correlation of expression}
\description{
  This is a sample dataset that contains 3,645 interactions corresponding to 
  genes with a correlation of expression higher than 70\%. Please note that 
  this file contains interactions between the genes from Sample_DE.csv. 
}
\format{A table containing 3,645 observations.}
\source{Sampled from a table generated as described in https://github.com/alberto-valdeolivas/RWR-MH/blob/master/Scripts_and_Files/Network_Generation/Generate_Coexpression_Network.R}
\keyword{datasets}