\name{Sample_DE.csv}
\docType{data}
\alias{Sample_DE}
\title{Sample of differential expression analysis results}
\description{
  This is a sample dataset that contains the log2(fold change) and the 
  False Discovery Rate (FDR) of 1,000 genes. 
}
\format{A table containing 1,000 observations.}
\source{Sampled from Banerji2017.csv 
  (https://github.com/elvanov/MOGAMUN-data/blob/master/PreprintFiles.zip).}
\keyword{datasets}