\name{2_Pathways_Sample.gr}
\docType{data}
\alias{2_Pathways_Sample}
\title{Sample of interactions from pathways}
\description{
  This is a sample dataset that contains 526 interactions corresponding to 
  genes associated to the same pathway. Please note that this file contains 
  interactions between the genes from Sample_DE.csv. 
}
\format{A table containing 526 observations.}
\source{Sampled from https://github.com/alberto-valdeolivas/RandomWalkRestartMH/blob/master/data/Pathway_Network.rda}
\keyword{datasets}