\name{Sample_NodesScore.csv}
\docType{data}
\alias{Sample_NodesScore}
\title{Sample of nodes scores}
\description{
  This is a sample dataset that contains the nodes scores of the 1,000 genes 
  from Sample_DE.csv. 
}
\format{A table containing 1,000 observations.}
\source{Generated using MOGAMUN.}
\keyword{datasets}